*==============================================================================
* Function:			ATCWord
* Purpose:			Performs an ATC() on whole words, not substrings
* Author:			Trevor Mansuy, with tweaks by Mike Potjer and Doug Hennig
* Last revision:	11/25/2016
* Parameters:		tcSearch     - the string to search for
*					tcString     - the string to search for a match in
*					tnOccurrence - the occurrence number (optional: if it isn't
*						passed, 1 is used)
* Returns:			the index of the first character of the match
* Environment in:	VBScript.RegExp can be instantiated
* Environment out:	none
* Notes:			ATC() returns the index of any match in the string, not
*						just words. This can be a problem if you want to
*						ATC('AS Country', '[Country Birth] AS Country_Birth'),
*						because we probably don't want just Country to match
*						with any other field name. 
*==============================================================================

lparameters tcSearch, ;
	tcString, ;
	tnOccurrence
local lnOccurrence, ;
	lcLeftBoundary, ;
	lcRightBoundary, ;
	lcDelimiters, ;
	lcSearch, ;
	lcString, ;
	lcSearchString, ;
	loRegExpr, ;
	loMatches, ;
	lnReturn, ;
	loMatch, ;
	lcMatch

* Ensure the necessary parameters were passed.

assert vartype(tcSearch) = 'C' and not empty(tcSearch) ;
	message 'Invalid search string passed'
assert vartype(tcString) = 'C' ;
	message 'Invalid string passed'

* Bug out if the string is empty.

if empty(tcString)
	return 0
endif empty(tcString)

* Get the occurrence number.

lnOccurrence = iif(vartype(tnOccurrence) = 'N', tnOccurrence, 1)

* This is here for future proofing. Right now, the function expects whatever we
* search for to begin and end with non-word characters, but just in case, save
* the boundary characters in variables so we can change them easily. Right now,
* we're using word-boundaries (the space between a word and a non-word
* character). We originally wanted to use non-word characters (\W) for this,
* but the match consumes the character, so side-by-side matches won't be found.
* The drawback of \b is if we do an ATC() for a string with a non-word
* character at the beginning or end, there's no guarantee there's a word
* boundary on the other side of that character.

lcLeftBoundary  = '\b'
lcRightBoundary = '\b'

* Strip the usual name delimiters since they interfere with the search.

lcDelimiters = '"[]`'
lcSearch     = chrtran(tcSearch, lcDelimiters, '')
lcString     = chrtran(tcString, lcDelimiters, '')

* Certain RegExp special characters prevent matches, so escape them.

lcSearchString = AddRegExEscape(lcSearch)
lcString       = AddRegExEscape(lcString)

* In the pattern below, the three sets of parentheses represent three match
* groups. The first match group, (LB|^), means we first match either a non-word
* character or the beginning of a string. Similarly, (RB|$) means at the end we
* match either a non-word character or the end of the string. This means we
* make a match in every situation except where the string in lcSearch is a
* substring of one of the strings in tcString.

loRegExp = createobject('VBScript.RegExp')
loRegExp.IgnoreCase = .T.
loRegExp.Global     = .T.
loRegExp.Pattern    = '(' + lcLeftBoundary + '|^)(' + lcSearchString + ')(' + ;
	lcRightBoundary + '|$)'

* Test the string and check how many matches were made. If it's less than the
* occurrence passed, return 0.

loMatches = loRegExp.Execute(lcString)
if loMatches.Count < lnOccurrence
	lnReturn = 0

* Retrieve the specified occurrence from the Item collection. If the match is
* the one we want, return its start location in the string.

else
	loMatch  = loMatches.Item(lnOccurrence - 1)
	lcMatch  = loMatch.Value
	lnReturn = iif(upper(lcSearch) = upper(lcMatch), loMatch.FirstIndex + 1, 0)
endif loMatches.Count < lnOccurrence
return lnReturn
