*==============================================================================
* Function:			StrTranWord
* Purpose:			Performs a STRTRAN on whole words, not substrings
* Author:			Trevor Mansuy
* Last revision:	06/24/2015
* Parameters:		tcString  - the string to search and replace in
*					tcSearch  - the string to search for 
*					tcReplace - the string to replace tcSearch with
* Returns:			the string with any replacements done
* Environment in:	VBScript.RegExp can be instantiated
* Environment out:	none
* Notes:			STRTRAN does a search and replace on anything in a string,
*						not just words. This can be a problem if you want to
*						STRTRAN('TABLE.TAX, TABLE.TAX_STATUS', 'TABLE.TAX',
*						'TABLE.TAX_A'), because TABLE.TAX is matched twice in
*						the string and you'd end up with TABLE.TAX_A,
*						TABLE.TAX_A_STATUS. This method won't match substrings,
*						only whole words
*==============================================================================

lparameters tcString, ;
	tcSearch, ;
	tcReplace
local loRegExpr, ;
	lcSearch, ;
	lcReplace, ;
	loMatchCollection, ;
	lcLeftBoundary, ;
	lcRightBoundary, ;
	lcResult

* Ensure the necessary parameters were passed.

assert vartype(tcSearch) = 'C' and not empty(tcSearch) ;
	message 'Invalid search string passed'
assert vartype(tcString) = 'C' ;
	message 'Invalid string passed'
assert vartype(tcReplace) = 'C' ;
	message 'Invalid replacement string passed'

* Bug out if the string is empty.

if empty(tcString)
	return tcString
endif empty(tcString)

* If the search string begins or ends with a non-word character, this causes
* the string match to fail. In some cases, however, we want the match and
* replacement to be successful (i.e. TableName.FieldName, replacing
* "TableName." with "Replacement."). To handle this, we check the left and
* right sides of the search string for non-word characters and use them instead
* of \W if they exist. We also have to remove these characters from the search
* string.

loRegExp = createobject('VBScript.RegExp')
loRegExp.IgnoreCase = .T.
loRegExp.Global     = .T.
loRegExp.Pattern    = '\W'
lcSearch            = tcSearch
lcReplace           = tcReplace
loMatchCollection   = loRegExp.Execute(left(tcSearch, 1))
if loMatchCollection.Count = 0
	lcLeftBoundary = '\W'
else
	lcLeftBoundary = AddRegExEscape(left(tcSearch, 1))
	lcSearch       = substr(lcSearch, 2)
endif loMatchCollection.Count = 0
loMatchCollection = loRegExp.Execute(right(tcSearch, 1))
if loMatchCollection.Count = 0
	lcRightBoundary = '\W'
else
	lcRightBoundary = AddRegExEscape(right(tcSearch, 1))
	lcSearch        = left(lcSearch, len(lcSearch) - 1)
endif loMatchCollection.Count = 0
   
* Certain RegExp special characters prevent matches, so escape them.

lcSearch = AddRegExEscape(lcSearch)

* In the pattern below, the three sets of parentheses represent three match
* groups. The first match group, (\W|^), means we first match either a non-word
* character or the beginning of a string. Similarly, (\W|$) means at the end we
* match either a non-word character or the end of the string. This means we do
* the replacement in every situation except where the string in lcSearch is a
* substring of one of the strings in tcString.

loRegExp.Pattern = '(' + lcLeftBoundary + ;
	iif(lcLeftBoundary = '\W', '|^)(', ')(') + lcSearch + ')(' + ;
	lcRightBoundary + iif(lcRightBoundary = '\W', '|$)', ')')

* Do the replacement. The $1 and $3 refer to the first and third capturing
* groups in our RegEx. We add these in case the non-word match consumes a
* boundary character such as a space, an operator in an expression, etc., and
* we want to put it back. If the first or third capturing groups captured a
* specific non-word character, ignore it during the replacement since a
* suitable character is assumed to be in the replacement string in this case.

lcResult = loRegExp.Replace(tcString, ;
	iif(lcLeftBoundary = '\W', '$1', '') + tcReplace + ;
	iif(lcRightBoundary = '\W', '$3', ''))
return lcResult
